/* match.c
 * Some functions dealing with strings.
 */

#include "weedlog.h"

/* wl_psize(buf);
 * A little function to get the number of valid characters in a pointer.
 */
int wl_psize(const char *buf) {
  register const char *s;
  int count = 0;

  for(s = buf; *s; s++) {
    count++;
  }
  return count;
}


/* lowercase(string);
 * Converts the string (string) to lowercase and returns the finished string.
 * Please note that this turns the origional string to lower case.
 * Example:
 * char *somestring, astring[] = "HeLLo";
 * sometime = (char*)astring;
 * lowercase(somestring);
 * astring turns from "HeLLo" to "hello" in the memory
 */
char *lowercase(char *string) {
  char *cp;

  for(cp = string; *cp; cp++) {
    *cp = (char)tolower(*cp);
  }
  return string;
}

/* checkisnum(string);
 * Checks to see whether the string is a number or not.
 */
int checkisnum(const char *string) {
  register const char *s = string;
  int firstloop = 1;

  while(*s) {
    if((*s != '0') && (*s != '1') && (*s != '2') && (*s != '3') && (*s != '4')
	&& (*s != '5') && (*s != '6') && (*s != '7') && (*s != '8')
	&& (*s != '9')) {
      if(firstloop) {
        if(*s != '-') {
	  return 0;
	}
	firstloop = 0;
      } else {
        return 0;
      }
    }
    s++;
  }
  return 1;
}

/* match(string, wild);
 * This function checks the string (string) against the (possibly?) wild
 * string (wild). The wild cards '*' and '?' are allowed. '*' matches any
 * number of characters. '?' matches against 1 character and cannot match
 * against less or more. Examples:
 * 'Heasdfallo' matchs h*llo, 'hellesallo' does not match h*llo
 * 'heheheeleleleeelllll!!!' matchs *h*ll*
 * 'he11o' matchs he??o, 'helo' does not match he??o
 * NOTE: This function is NOT case insensitive.
 *
 * Returns 1 on match.
 * Returns 0 on non-match.
 */
int match(const char *string, const char *wild) {
  register const char *s = string, *w = wild;
  int star = 0, n = 0;

  do {
top:
    if(*w == '\\') {	/* This is for escaping '*' & '?' in the wild string. */
      w++;		/* Go to the next char and see if it is is a '*' */
      if((*w == '*') || (*w == '?')) {	/* or a '?', otherwise ignore it's */
        if(tolower(*s) != tolower(*w))	/* exsistance. */
	  return 0;
	w++;
	s++;
      }
    }
    if(*w == '*') {	/* Checks for multi-char wild card. */
      while(*w && (*w == '*')) { /* Clear out multiple stars because they are stupid, */
        w++;		/* '******' does exactly the same as '*'. */
      }
      if(!*w)		/* Last char is a '*' so we should asume the rest */
        return 1;	/* is a match. */
      star = 1;
    }
    if(*w == '?') {	/* Checks for single char wild card. */
      while(*w && (*w == '?')) {
	w++;		/* If '?' jump to the next char to check against. */
	if(!star) {	/* Make sure there isn't a '*' and a '?' because a */
	  s++;		/* '*' preceeding '?' nullifies '?', otherwise move */
	}
      }
    }			/* the string we are checking forward 1 char. */
    if(star) {		/* This is set when a '*' is found. */
star:
      if(!*w)		/* Redundant error check: asume last char is wild. */
        return 1;
      if(!*s)		/* Would have returned 1 if last char was wild, so */
        return 0;	/* we asume a missmatch if nothing to check against. */
      while(*s && (tolower(*s) != tolower(*w))) {
        s++;		/* '*' is any number of chars so go */
      }
      if(!*s && *w)	/* There is nothing left to check but we still have */
        return 0;	/* stuff to check against so assume a missmatch. */
      if(tolower(*s) == tolower(*w)) {
        int need = 1, got = 1;
        while(tolower(*w) == tolower(w[need])) {
	  need++;
	}
        while(tolower(s[got]) == tolower(*w)) {
	  got++;
	}
	if(got >= need) {
	  for(n = 0; n < need; n++) {
	    *w++;
	  }
	  for(n = 0; n < got; n++) {
	    *s++;
	  }
	  if(!(*s && *w)) {
	    return 1;
	  }
	} else {
	  for(n = 0; n < got; n++) {
	    *s++;
	  }
	  goto star;
	}
      }
      star = 0;
    }
    if(tolower(*s) != tolower(*w)) {
      if((*w == '*') || (*w == '?')) /* double check for wildcards */
        goto top;
      else return 0;	/* We have a missmatch so return 0. */
    }
    while((*s && *w) && (tolower(*s) == tolower(*w))) {
      w++;
      s++;
      if(!(*s && *w))	/* If we are here the previous chars matched so if */
        return 1;	/* there are none left we asume a match. */
    }
  }
  while(*s);

  if(!(*s && *w)) {
    return 1;
  }
  return 0;
}
