#include "weedlog.h"

struct udppacket {
  struct wl_iphdr {
#if __BYTE_ORDER == __LITTLE_ENDIAN
    u_char	ihl:4,
		version:4;
#elif __BYTE_ORDER == __BIG_ENDIAN
    u_char	version:4,
		ihl:4;
#else
#error	"Please fix <asm/byteorder.h>"
#endif
    u_char	tos;
    u_short	tot_len;
    u_short	id;
    u_short	frag_off;
    u_char	ttl;
    u_char	protocol;
    u_short	check;
    u_long	saddr;
    u_long	daddr;
  }ip;
  struct wl_udphdr {
#ifdef __FAVOR_BSD
    u_short	uh_sport;           /* source port */
    u_short	uh_dport;           /* destination port */
    u_short	uh_ulen;            /* udp length */
    u_short	uh_sum;             /* udp checksum */
#else
    u_short	source;
    u_short	dest;
    u_short	len;
    u_short	check;
#endif /* __FAVOR_BSD */
  }udp;
};

int parse_udp(const u_char *packbuff, int verbose) {
  int pktsize=0;
  char newbuf[512], suspip[24], psizebuf[24], tempflags[128], ipflags[128],
      saddress[82], daddress[88], dport[64], sport[64], csums[128];
  struct udppacket *pkt = (struct udppacket*)packbuff;

  if(pkt->ip.ihl != 5)
    snprintf(suspip, sizeof(suspip), " Suspicious IP Options: ");
  else
    snprintf(suspip, sizeof(suspip), " ");

  if((pktsize = ntohs(pkt->ip.tot_len)) > 1024)
    snprintf(psizebuf, sizeof(psizebuf), "%4.02f KBytes", (float)pktsize / 1024.0);
  else
    snprintf(psizebuf, sizeof(psizebuf), "%4.02f Bytes", (float)pktsize);

  if(pkt->ip.saddr) {
    struct in_addr s;
    s.s_addr = pkt->ip.saddr;
    snprintf(saddress, sizeof(saddress), "%s[%s]",
	hostlookup(pkt->ip.saddr), inet_ntoa(s));
  } else {
    snprintf(saddress, sizeof(saddress), "nowhere it would seem");
  }
  if(pkt->ip.daddr) {
    struct in_addr d;
    d.s_addr = pkt->ip.daddr;
    snprintf(daddress, sizeof(daddress), " to %s[%s],",
	hostlookup(pkt->ip.daddr), inet_ntoa(d));
  } else {
    snprintf(daddress, sizeof(daddress), ",");
  }

  if(verbose >= 3) {
    snprintf(ipflags, sizeof(ipflags), "  IP_FLAGS:");
    if(pkt->ip.version > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s VER=%i", ipflags,
	  pkt->ip.version);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.protocol > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s PROTO=%i", ipflags,
	  pkt->ip.protocol);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.frag_off > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s FRAG_OFF=%i", ipflags,
	  pkt->ip.frag_off);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.tos > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s TOS=%i", ipflags,
	  pkt->ip.tos);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.id > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s ID=%i", ipflags,
	  pkt->ip.id);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.ttl > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s TTL=%i", ipflags,
	  pkt->ip.ttl);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
    if(pkt->ip.check > 0) {
      snprintf(tempflags, sizeof(tempflags), "%s CHECK=%i", ipflags,
	  pkt->ip.check);
      snprintf(ipflags, sizeof(ipflags), "%s", tempflags);
    }
  } else
    snprintf(ipflags, sizeof(ipflags), " ");

#ifdef __FAVOR_BSD
  snprintf(dport, sizeof(dport), "%d(%s)", ntohs(pkt->udp.uh_dport),
      servlookup(pkt->udp.uh_dport,17));
  snprintf(sport, sizeof(sport), "%d(%s)", ntohs(pkt->udp.uh_sport),
      servlookup(pkt->udp.uh_sport,17));
  if(verbose >= 2)
    snprintf(csums, sizeof(csums), "  UH_Len: %u  Checksum: %u",
	ntohs(pkt->udp.uh_ulen), pkt->udp.uh_sum);
  else if(verbose >= 1)
    snprintf(csums, sizeof(csums), "  UH_Len: %u", ntohs(pkt->udp.uh_ulen));
  else
    snprintf(csums, sizeof(csums), " ");
#else
  snprintf(dport, sizeof(dport), "%d(%s)", ntohs(pkt->udp.dest),
      servlookup(pkt->udp.dest,17));
  snprintf(sport, sizeof(sport), "%d(%s)", ntohs(pkt->udp.source),
      servlookup(pkt->udp.source,17));
  if(verbose >= 2)
    snprintf(csums, sizeof(csums), "  UH_Len: %u  Checksum: %u",
	ntohs(pkt->udp.len), pkt->udp.check);
  else if(verbose >= 1)
    snprintf(csums, sizeof(csums), "  UH_Len: %u", ntohs(pkt->udp.len));
  else
    snprintf(csums, sizeof(csums), " ");
#endif /* __FAVOR_BSD */

  snprintf(newbuf, sizeof(newbuf), "UDP: %sPort %s packet from %s%s Size: %s  "
      "Source Port: %s%s%s", suspip, dport, saddress, daddress,
      psizebuf, sport, csums, ipflags);

  print_log("%s\n", newbuf);

  return 0;
}
