/* sockets.c
 * Stuff for dealing with sockets such as creating them and reading them.
 */

#include "weedlog.h"

void setup_sockets(int socktypes, wl_verb verb) {
  int s, tvl = 0;

  if(TCPIsSet(socktypes)) {
    /* the tcp socket */
    if((s = socket(AF_INET, SOCK_RAW, 6)) < 0) {
      print_log_err("Error creating TCP socket. Errno: %s\n", strerror(errno));
    }
    setsock_nonblock(s,TRUE);
    if(verb.tcp) tvl = verb.tcp;
    else if(verb.global) tvl = verb.global;
    else tvl = 0;
    addsocket(s, SOCKTYPE_TCP, tvl);
  }
  if(UDPIsSet(socktypes)) {
    /* the udp socket */
    if((s = socket(AF_INET, SOCK_RAW, 17)) < 0) {
      print_log_err("Error creating UDP socket. Errno: %s\n", strerror(errno));
    }
    setsock_nonblock(s, TRUE);
    if(verb.udp) tvl = verb.udp;
    else if(verb.global) tvl = verb.global;
    else tvl = 0;
    addsocket(s, SOCKTYPE_UDP, tvl);
  }
  if(ICMPIsSet(socktypes)) {
    /* the icmp socket */
    if((s = socket(AF_INET, SOCK_RAW, 1)) < 0) {
      print_log_err("Error creating ICMP socket. Errno: %s\n", strerror(errno));
    }
    setsock_nonblock(s, TRUE);
    if(verb.icmp) tvl = verb.icmp;
    else if(verb.global) tvl = verb.global;
    else tvl = 0;
    addsocket(s, SOCKTYPE_ICMP, tvl);
  }
  if(IGMPIsSet(socktypes)) {
    /* the icmp socket */
    if((s = socket(AF_INET, SOCK_RAW, 2)) < 0) {
      print_log_err("Error creating IGMP socket. Errno: %s\n", strerror(errno));
    }
    setsock_nonblock(s, TRUE);
    if(verb.igmp) tvl = verb.igmp;
    else if(verb.global) tvl = verb.global;
    else tvl = 0;
    addsocket(s, SOCKTYPE_IGMP, tvl);
  }
}

int addsocket(int sock, int type, int verbl) {
  wl_socket *socktoadd, *tmp = socketlist;

  if((socktoadd = (wl_socket*)malloc(sizeof(*socktoadd))) < 0) {
    print_log("malloc: %s", strerror(errno));
    return -1;
  }
  socktoadd->next = (wl_socket*)NULL;
  socktoadd->prev = (wl_socket*)NULL;
  socktoadd->fd = sock;
  socktoadd->type = type;
  socktoadd->verbose = verbl;
  if(socketlist == (wl_socket*)NULL) {
    socketlist = socktoadd;
    return 1;
  } else {
    while(tmp) {
      if(!tmp->next) {
        tmp->next = socktoadd;
	socktoadd->prev = tmp;
	return 1;
      } else tmp = tmp->next;
    }
  }
  return 0;
}
/*
void setsock_promisc(int fd, struct ifreq ifr) {
  ifr.ifr_flags |= IFF_PROMISC;
  if(ioctl(fd, SIOCSIFFLAGS, &ifr) < 0) {
    print_log("SIOCSIFFLAGS: %s", strerror(errno));
  } else {
    ifr.ifr_flags &= ~IFF_PROMISC;
    * tell the program to set the device non-promiscuous when it exit's *
    atexit(ioctl(fd, SIOCSIFFLAGS, &ifr));
  }
}
*/
int setsock_nonblock(int fd, int dontblock) {
  int opts;

  if(dontblock) {
    if((opts = fcntl(fd,F_GETFL)) < 0) {
      print_log("fcntl (F_GETFL): %s", strerror(errno));
      return -1;
    }
    opts |= O_NONBLOCK;
    if(fcntl(fd,F_SETFL,opts) < 0) {
      print_log("fcntl (F_SETFL): %s", strerror(errno));
      return -1;
    }
  } else {
    if((opts = fcntl(fd,F_GETFL)) < 0) {
      print_log("fcntl (F_GETFL): %s", strerror(errno));
      return -1;
    }
    opts &= ~O_NONBLOCK;
    if(fcntl(fd,F_SETFL,opts) < 0) {
      print_log("fcntl (F_SETFL): %s", strerror(errno));
      return -1;
    }
  }
  return 0;
}

int check_for_packet() {
  u_char *pbuff;
  wl_socket *checksocks = socketlist;
  struct sockaddr from;
  int amountread = 0, fromlen, fdproto = 0;

  if(!checksocks)
    return 0;

  if((pbuff = (u_char*)malloc(PACKET_BUF_SIZE)) < 0){
    print_log("check_for_packet: malloc: %s", strerror(errno));
    return -1;
  }
  fromlen = sizeof(from);
  while(checksocks) {
    if(checksocks->fd) {
      GetProto(checksocks->type,fdproto);
      if((amountread = recvfrom(checksocks->fd, (u_char*)pbuff,
	  PACKET_BUF_SIZE, 0, &from, &fromlen)) < 0) {
	if(errno != 11) { /* EAGAIN a.k.a. EWOULDBLOCK */
	  print_log("error: Received error %i (%s) while attempting to read"
	      " from file descriptor %i (proto: %i).\n", errno,
	      strerror(errno), checksocks->fd, fdproto);
	} else {
	  if(debug >= 5)
	    print_log("debug: Got nothing on file descriptor %i (proto: %i).\n",
		checksocks->fd, fdproto);
	}
      } else {
	if(debug >= 5) {
	  print_log("debug: Got something on file descriptor %i (proto: %i).\n",
	      checksocks->fd, fdproto);
	  print_log("debug: read %i, fromlen == %i, from.data "
	      "== %s, from.sa_family == %i\n", amountread, fromlen,
	      from.sa_data, from.sa_family);
	}
	if(ICMPIsSet(checksocks->type)) {
	  parse_icmp(pbuff, checksocks->verbose);
	} else if(IGMPIsSet(checksocks->type)) {
	  parse_igmp(pbuff, checksocks->verbose);
	} else if(TCPIsSet(checksocks->type)) {
	  parse_tcp(pbuff, checksocks->verbose);
	} else if(UDPIsSet(checksocks->type)) {
	  parse_udp(pbuff, checksocks->verbose);
	}
      }
    }
    checksocks = checksocks->next;
  }
  if(pbuff)
    free(pbuff);
  return 0;
}
