/* weedlog.c
 * Contains general stuff needed to run the program.
 */

#include "weedlog.h"

int socktypes = 0, bequiet = 0, daemon_mode=0, console_mode=0, gverb=0,
    tcpverb=0, icmpverb=0, udpverb=0, igmpverb=0;

/* go_background();
 * Sends the program into the background (ie daemon mode).
 * Returns the new process id.
 */
int go_background() {
  int fs = -1;

  switch((fs = fork())) {
    case -1:
      return (-1);
    case 0:
      break;
    default:
      exit(0);
  }

  if(setsid() == -1)
    return(-1);

  (void)chdir("/");

  return fs;

}

int showhelp(char *name) {
  fprintf(stdout, "weedlogd version %s\n", VERSION);
  fprintf(stdout, "Option parts surrounded by [] are optional.\n");
  fprintf(stdout, "Option parts surrounded by <> are required.\n");
  fprintf(stdout, "Synopsis:\n");
  fprintf(stdout, "  %s [-F <file>] [-I] [-T] [-U] [-V<p> [#]] [-b] [-d [#]] [-f] [-h] [-v [#]] [-q]\n", name);
  fprintf(stdout, "    OR\n");
  fprintf(stdout, "  %s -[F][I][T][U][V<p>][b][d][f][h][v][q] <file> [procotol verbose #] [debug #] [verbose #]\n", name);
  fprintf(stdout, "Command Line Options:\n");
  fprintf(stdout, " Protocols:\n");
  fprintf(stdout, "  -I\t\t: Watch for ICMP packets.\n");
  fprintf(stdout, "  -T\t\t: Watch for TCP packets.\n");
  fprintf(stdout, "  -U\t\t: Watch for UDP packets.\n");
  fprintf(stdout, " Output Options:\n");
  fprintf(stdout, "  -F <file>\t: Use <file> instead of syslog when in Background/Daemon mode.\n");
  fprintf(stdout, "  -b\t\t: Use Background/Daemon mode.\n");
  fprintf(stdout, "  -f\t\t: Use Foreground/Console mode.\n");
  fprintf(stdout, " Verbose Options:\n");
  fprintf(stdout, "  -V<p> [#]\t: Set verbose level.\n");
  fprintf(stdout, "\t\t- \'<p>\' should be replaced by a letter signifying a protocol.\n");
  fprintf(stdout, "\t\t - Valid protocols are 'I', 'T' and 'U'.\n");
  fprintf(stdout, "  -d [#]\t: Use debug mode level #.\n");
  fprintf(stdout, "\t\t- effective debug mode levels: 5\n");
  fprintf(stdout, "  -v [#]\t: Set verbose level.\n");
  fprintf(stdout, "\t\t- effective verbose mode levels: 3\n");
  fprintf(stdout, "  READ THIS: Giving the options a numeric argument is optional. If you use the\n"
	"  compressed format (-dVTVIVUv...) and wish to give numeric arguments to some,\n"
	"  but not all, of these options then the ones you wish to give numeric arguments\n"
	"  to must come first or the others will steal their numbers.\n");
  fprintf(stdout, " Misc. Options:\n");
  fprintf(stdout, "  -h\t\t: This help message.\n");
  fprintf(stdout, "  -q\t\t: Be quiet when starting up (good for starting at boot).\n");
  fprintf(stdout, "\nExamples:\n");
  fprintf(stdout, "  See the README file or manual page for examples.\n");
  fprintf(stdout, "\nNotes:\n");
  fprintf(stdout, "  - The options -f and -F may be used together to print to "
      "both stdout and the\n  given filename at the same time. You cannot send to"
      " a file and to syslog at\n  the same time (if running in the background).\n");
  exit(0);
  return 0;
}

/* parse_args(argc, argv);
 * Parses the command line arguments.
 */
int parse_args(int argc, char *argv[]) {
  register const char *s;
  int n = 1;

  while(argv[n]) {
    s = argv[n];
    if(*s == '-') {
      s++;
      while(*s) {
	if(*s == 'F') {
	  register const char *filepath;
	  char *mycwd, *fname;
	  n++;
	  filepath = argv[n];
	  if(!filepath) goto NEXT;
	  if((mycwd = (char*)malloc(PATH_MAX)) < 0) {
	    print_log("malloc: %s\n", strerror(errno));
	    goto NEXT;
	  }
	  if((fname = (char*)malloc(PATH_MAX)) < 0) {
	    print_log("malloc: %s\n", strerror(errno));
	    goto NEXT;
	  }
	  if(*filepath == '/') {
	    snprintf(fname, PATH_MAX, "%s", filepath);
	  } else {
	    getcwd(mycwd, PATH_MAX);
	    snprintf(fname, PATH_MAX, "%s/%s", mycwd, filepath);
	  }
	  if((outfile = open(fname, O_WRONLY|O_CREAT|O_APPEND, S_IRUSR|S_IWUSR|S_IRGRP)) < 0) {
	    print_log("error: could not open %s (%s)\n", fname, strerror(errno));
	  }
	  if(fname)
	    free(fname);
	  if(mycwd)
	    free(mycwd);
	  goto NEXT;
	}
	if(*s == 'I') {
	  SetICMP(socktypes);
	  goto NEXT;
	}
	if(*s == 'G') {
	  SetIGMP(socktypes);
	  goto NEXT;
	}
	if(*s == 'T') {
	  SetTCP(socktypes);
	  goto NEXT;
	}
	if(*s == 'U') {
	  SetUDP(socktypes);
	  goto NEXT;
	}
	if(*s == 'V') {
	  s++;
	  if(*s == 'I') {
	    if(argv[n+1]) {
	      if(checkisnum(argv[n+1])) {
		n++;
		icmpverb = atoi(argv[n]);
	      } else icmpverb++;
	    } else icmpverb++;
	  }
	  if(*s == 'G') {
	    if(argv[n+1]) {
	      if(checkisnum(argv[n+1])) {
		n++;
		igmpverb = atoi(argv[n]);
	      } else igmpverb++;
	    } else igmpverb++;
	  }
	  if(*s == 'T') {
	    if(argv[n+1]) {
	      if(checkisnum(argv[n+1])) {
		n++;
		tcpverb = atoi(argv[n]);
	      } else tcpverb++;
	    } else tcpverb++;
	  }
	  if(*s == 'U') {
	    if(argv[n+1]) {
	      if(checkisnum(argv[n+1])) {
		n++;
		udpverb = atoi(argv[n]);
	      } else udpverb++;
	    } else udpverb++;
	  }
	  goto NEXT;
	}
	if(*s == 'b') {
	  daemon_mode = 1;
	  goto NEXT;
	}
	if(*s == 'd') {
	  if(argv[n+1]) {
	    if(checkisnum(argv[n+1])) {
	      n++;
	      debug = atoi(argv[n]);
	    } else debug++;
	  } else debug++;
	  goto NEXT;
	}
	if(*s == 'f') {
	  console_mode = 1;
	  goto NEXT;
	}
	if(*s == 'h') {
	  showhelp(argv[0]);
	  goto NEXT;
	}
	if(*s == 'v') {
	  if(argv[n+1]) {
	    if(checkisnum(argv[n+1])) {
	      n++;
	      gverb = atoi(argv[n]);
	    } else gverb++;
	  } else gverb++;
	  goto NEXT;
	}
	if(*s == 'q') {
	  bequiet++;
	  goto NEXT;
	}
/*
	if(*s == '') {
	  goto NEXT;
	}
*/
NEXT:
	s++;
      }
    }
    n++;
  }

  if(daemon_mode == 1 && console_mode == 1)
    console_mode = 0;
  if(daemon_mode == 0 && console_mode == 0)
    daemon_mode = 1;
  return 0;
}

int main(int argc, char *argv[]) {
  int bg;
  time_t t;
  wl_verb verblev;

  progname = argv[0];
  wlpid = getpid();
  logmode = 0;

  parse_args(argc, argv);

  setuid(0);
  if(geteuid() != 0)
    print_log("init: This program requires root privledges. (%s)\n",
	strerror(errno));

  verblev.global = gverb;
  verblev.icmp = icmpverb;
  verblev.igmp = igmpverb;
  verblev.tcp = tcpverb;
  verblev.udp = udpverb;

  if(!bequiet) {
    t = time(NULL);
    print_log("init: %s started at %s", progname, ctime(&t));
  }

  if(debug >= 2) {
    fprintf(stdout, "debug level: %i\nverbose levels:\n  global: %i\n  icmp: %i"
	"\n  tcp: %i\n  udp: %i\n", debug, verblev.global, verblev.icmp,
	verblev.tcp, verblev.udp);
  }

  if(daemon_mode) {
    if(!bequiet)
      printb_log(FALSE, "init: Entering background mode.\n");
    if(!outfile) {
      char *syslogname;
      if((syslogname = (char*)malloc(PATH_MAX)) < 0) {
	print_log("malloc: %s", strerror(errno));
      }
      sprintf(syslogname, "%s[%i]", progname, wlpid);
      openlog(syslogname, 0, LOG_DAEMON);
      logmode = LOGMODE_SYSL;
      if(debug)
	print_log("Testing syslog status under name %s\n", syslogname);
    } else {
      logmode = LOGMODE_FILE;
    }
    if((bg = go_background()) < 0) {
      print_log_err("error: fork() failed (%s), exiting.\n", strerror(errno));
      return 1;
    } else {
      wlpid = getpid();
      if(!bequiet)
	print_log("init: Background mode entered. PID changed to %i.\n", wlpid);
      if(logmode == LOGMODE_SYSL) {
	char *syslogname;
	if((syslogname = (char*)malloc(PATH_MAX)) < 0) {
	  print_log("malloc: %s", strerror(errno));
	}
	sprintf(syslogname, "%s[%i]", progname, wlpid);
	closelog();
	openlog(syslogname, 0, LOG_DAEMON);
	logmode = LOGMODE_SYSL;
	if(debug)
	  print_log("Testing syslog status under name %s\n", syslogname);
      }
    }
  } else wlpid = getpid();

  if(outfile && !daemon_mode)
    logmode = LOGMODE_FFIL;

  setup_sockets(socktypes, verblev);
  if(!socketlist) {
    printb_log(FALSE, "error: No sockets to watch, exiting.\n");
    return 1;
  }

  for(;;) {
    if(debug >= 5)
      printb_log(FALSE, "debug: Checking for packets\n");
    check_for_packet();
    if(debug >= 5)
      printb_log(FALSE, "debug: Sleeping.\n");
    usleep(100);
  }

  return 0;
}

